; RlxPsXref
; attach xref in paperspace
; Source: https://www.cadtutor.net/forum/topic/66789-lisp-to-attach-xref-to-multiple-drawings/

; 2020-07-28 CAD Concepts
; https://www.cadconcepts.co.nz
; Updated to suit BricsCAD V20 + add options to place Xref on a specific layer

(defun c:RlxPsXref (/ z_xref app adoc odbs odbx v xref folder dwg xr oLayer lay)
  (vl-load-com)
  (setq laynm "z_xref"); set target layer
  ;In BricsCAD you can use dos_getdir
;  (defun _getfolder (m / sh f r)
;    (setq sh (vla-getinterfaceobject (vlax-get-acad-object) "Shell.Application") f (vlax-invoke-method sh 'browseforfolder 0 m 0))
;    (vlax-release-object sh) (if f
;      (progn
;        (setq r (vlax-get-property (vlax-get-property f 'self) 'path)) (if (wcmatch r "*\\") r (strcat r "\\"))
;      )
;    )
;  )
  ; RLXref_SetPathType
  ; This function is intended to set an AutoCAD registry location, it would need to be updated for BricsCAD
  ; BricsCAD ignores the XATTACH dialogue path setting with vla-AttachExternalReference in any case
  ; i is 0 (absolute), 1 (relative) of 2 (no) -xref path
;  (defun RLXref_SetPathType (i)
;    (vl-registry-write (strcat "HKEY_CURRENT_USER\\" (vlax-product-key) "\\Profiles\\" (getvar "cprofile") "\\Dialogs\\XattachDialog") "PathType" i)
;  )
  
  (setq odbs "ObjectDBX.AxDbDocument" v (substr (getvar 'acadver) 1 2) adoc (vla-get-activedocument (setq app (vlax-get-acad-object))))
  ; Don't call this function. BricsCAD doesn't use XATTACH path setting with vla-AttachExternalReference
  ;(RLXref_SetPathType 1)
  (cond
    ((vl-catch-all-error-p (setq odbx (vl-catch-all-apply 'vla-getinterfaceobject (list app (if (< (atoi v) 16) odbs (strcat odbs "." v))))))
                           (princ "\nObject DBX interface not created!")
    )
    ((not (setq xref (getfiled "Select Xref to attach" "" "dwg" 0))) (alert "No Xref was selected"))
    ;((setq folder (_getfolder "Select folder with drawings to attach xref to"))
     ((setq folder (dos_getdir "Xref Attach" (getvar 'dwgprefix) "Select folder with drawings to attach xref to" 0))
      ; could use dos_getfilem to allow the user to select multiple .dwg files instead
      (foreach dwg (vl-directory-files folder "*.dwg" 0)
        (setq dwg (strcat folder dwg))
        (if (vl-catch-all-error-p (vl-catch-all-apply 'vla-open (list odbx dwg)))
          (princ (strcase (strcat "\nError opening: " dwg)))
          (progn
            (princ (strcat "\nOpening: " dwg))
            ; attach xref
            (if (vl-catch-all-error-p (setq xr (vl-catch-all-apply 'vla-AttachExternalReference
                                                                   (list (vla-get-ModelSpace odbx) xref (vl-filename-base xref) (vlax-3d-point 0 0 0) 1 1 1 0 :vlax-false)
                                               )
                                      )
                )
              (princ (vl-catch-all-error-message xr))
              (progn
                ;Put Xref on desired Layer
                (setq oLayer (vla-get-layers odbx)) ; retrieve the layer object
                (vla-add oLayer laynm) ; Make sure the layer exists
                (vla-put-layer xr laynm) ; Put the xref on the desired layer
                ;Move Xref to paperspace Layout
                ;This part copies the xref to the current layout in the drawing
                (vlax-for lay (vla-get-layouts odbx)
                  (if (/= "MODEL" (strcase (vla-get-name lay))) (vlax-invoke odbx 'copyobjects (list xr) (vla-get-block lay)))
                )
                ;Xref in model space is then deleted
                (vla-delete xr)
              )
            )
            ; save drawing
            (vla-saveas odbx (vla-get-name odbx))
          )
        )
      )
    )
  )
  (princ)
)
